import java.io.*;
import java.security.*;
import java.net.*;


public class ClientSession implements Session {
    private Socket s;
    private ObjectOutputStream os;
    private ObjectInputStream is;
    private BufferedReader textIn;

    private String username;
    private Crypto crypto;

    // This field is initialized during authentication
    private Key kSession;

    // Additional fields here

    ClientSession(Socket s, String username) {
	this.s = s;
	this.username = username;
	this.crypto = new Crypto();
	try {
	    textIn = new BufferedReader(new InputStreamReader(System.in));
	    OutputStream out =  s.getOutputStream();
	    this.os = new ObjectOutputStream(out);
	    InputStream in = s.getInputStream();
	    this.is = new ObjectInputStream(in);
	} catch (IOException e) {
	    e.printStackTrace();
	}
    }

    // This method authenticates the user and establishes a session key.
    public boolean authenticateClient() {
	Key kAuth;
	try {
	    kAuth = (Key)Disk.load(username + ".shared");
	} catch (IOException e) {
	    System.out.println("# could not find key for user: " + username);
	    return false;
	}

	try {
	    // Construct and send Message 1 of the protocol
	    System.out.println("# sending Message 1...");
	    Nonce n1 = new Nonce();
	    Message1 m1 = new Message1(username, (Serializable)n1);
	    os.writeObject(m1);

	    // Wait for Message 2 of the protocol
	    Message2 m2 = (Message2)is.readObject();
	    System.out.println("# received Message 2...");

	    // Check the returned nonce
	    Nonce n1rec = (Nonce)crypto.decryptRijndael(m2.response, kAuth);
	    if (!n1.equals(n1rec)) {
		// nonces don't match
		System.out.println("# server returned wrong nonce");
		return false;
	    }
	    System.out.println("# nonce check succeeded...");

	    // Create the session key
	    kSession = crypto.makeRijndaelKey();

	    // Construct and send Message 3 of the protocol
	    Nonce n2 = (Nonce)m2.nonce;
	    Message3 m3 = new Message3(crypto.encryptRijndael(n2, kAuth),
				       crypto.encryptRijndael(kSession, kAuth));
	    os.writeObject(m3);

	    // Protocol completed successfully
	    System.out.println("# authentication complete.");
	    return true;
	} catch (Exception e) {
	    return false;
	}
    }

    public boolean doTransaction() {
	try {

	    String msg = textIn.readLine();

	    // Encrypt and send Message 4
	    Message4 m4 = new Message4(crypto.encryptRijndael(msg, kSession));
	    os.writeObject(m4);

	    if (msg.equals("done")) return false;
	    return true;
	} catch(Exception e) {
	    System.out.println("# error in message send");
	    return false;
	} 
    }
}
