import java.net.*;
import java.io.*;
import java.security.*;

public class ServerSession implements Session, Runnable {
    private Socket s;
    private ObjectOutputStream os;
    private ObjectInputStream is;

    private Crypto crypto;

    // These fields are initialized during authentication
    private Key kSession;
    private String username;

    // Add additional fields you need here

    ServerSession(Socket s)
	throws IOException
    {
	this.s = s;
	OutputStream out =  s.getOutputStream();
	this.os = new ObjectOutputStream(out);
	InputStream in = s.getInputStream();
	this.is = new ObjectInputStream(in);
	this.crypto = new Crypto();
    }

    public void run() {
	try {
	    if (authenticateClient()) {
		while (doTransaction()) {
		    // loop
		}
	    }
	    is.close();
	    os.close();
	} 
	catch (Exception e) {
	    e.printStackTrace();
	}
    }
    
    // Interacts with an Client to 
    // (1) Authenticate the user
    // (2) If the user is valid, establish session key and any
    //     additional information needed for the protocol.
    // (3) Returns true if the user authentication succeeds, false otherwise
    public boolean authenticateClient() {
	try {
	    // Wait for Message 1 of the protocol
	    Message1 m1 = (Message1)is.readObject();
	    username = m1.username;
	    Serializable n1 = m1.nonce;
	    System.out.println("# authenticating user: " + username);

	    // Load the shared key for username
	    Key kAuth;
	    try {
		kAuth = (Key)Disk.load(username + ".shared");
	    } catch (IOException e) {
		System.out.println("# could not find key for user: " + username);
		return false;
	    }
	    
	    // Construct and send Message 2 of the protocol
	    Nonce n2 = new Nonce();
	    Message2 m2 = new Message2(crypto.encryptRijndael(n1, kAuth), n2);
	    System.out.println("# sending Message 2...");
	    os.writeObject(m2);


	    // Wait for Message 3 of the protocol
	    Message3 m3 = (Message3)is.readObject();
	    if (n2.equals((Nonce)crypto.decryptRijndael(m3.response, kAuth))) {
		System.out.println("# authentication succeeded.");
		kSession = (Key)crypto.decryptRijndael(m3.kSessionE, kAuth);
		return true;
	    } else {
		System.out.println("# authentication failed.");
		return false;
	    }

	} catch (Exception e) {
	    return false;
	}
    }

    // Interacts with an Client to 
    // (1) Accept a string encrypted with kSession
    // (2) Print the string, tagged with the username
    //  or end the session if the message received is just the string "done"
    public boolean doTransaction() {
	try {
	    Message4 m4 = (Message4)is.readObject();
	    String msg = (String)crypto.decryptRijndael(m4.data, kSession);
	    if (msg.equals("done")) {
		System.out.println("# " + username + "'s session finished");
		return false;
	    } else {
		System.out.println(username + ": " + msg);
		return true;
	    }
	} catch (Exception e) {
	    // ignore errors that arise during transmission
	    return true;
	}
    }
}

