import java.security.*;
import cryptix.provider.rsa.*;
import cryptix.provider.key.*;

/* This program is used by Server administrators to creates keys:

     The first argument <username> specifies the user associated with the key     

Key or KeyPair and writes it to disk */
public class KeyGen {

    public static void main(String[] args) {
	if (args.length != 2) {
	    System.out.println("Usage: java KeyGen <username> [rsa | rij]");
	    System.out.println("If the second argument is 'rsa', KeyGen creates \n" +
			       "files <username>.pub and <username>.priv \n" + 
                               "containing the RSA public and private keys. \n");
	    System.out.println("If the second argument is 'rij', KeyGen creates file \n" +
			       "<username>.shared containing a Rijndael Key object.");
	    System.exit(0);
	}
	try {
	    SecureRandom sr = new SecureRandom();
	    String username = args[0];
	    if (args[1].equals("rsa")) {
		BaseRSAKeyPairGenerator kpg = new BaseRSAKeyPairGenerator();
		kpg.initialize();
		KeyPair kp = kpg.generateKeyPair();
		Disk.save(kp.getPublic(), username + ".pub");
		Disk.save(kp.getPrivate(), username + ".priv");
	    } else if (args[1].equals("rij")) {
		RijndaelKeyGenerator kg = new RijndaelKeyGenerator();
		kg.initialize(sr);
		Key key = kg.generateKey();
		Disk.save(key, username + ".shared");
	    } else {
		System.out.println("Second argument must be 'rsa' or 'rij'");
		System.exit(0);
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
}
